@@ -8,8 +8,8 @@ typedef struct {
     PyObject_HEAD
     client_t *client;
     PyGreenlet *greenlet;
-    PyObject *args;
-    PyObject *kwargs;
+    PyObject *args;         //greenlet.switch value
+    PyObject *kwargs;       //greenlet.switch value
     uint8_t suspended;
     uint8_t resumed;
 } ClientObject;
@@ -25,4 +25,5 @@ setup_client(void);
 inline int 
 CheckClientObject(PyObject *obj);
 
+
 #endif
