@@ -1,5 +1,6 @@
 #include "response.h"
 #include "log.h"
+#include "socket.h"
 
 #define CRLF "\r\n"
 #define DELIM ": "
@@ -56,11 +57,12 @@ void
 send_error_page(client_t *client)
 {
     shutdown(client->fd, SHUT_RD);
-    if(client->header_done){
+    if(client->header_done || client->response_closed){
         //already sended response data
         //close connection
         return;
     }
+
     int status = client->bad_request_code;
     int r = status < 0 ? status * -1:status;
     client->status_code = r;
@@ -83,25 +85,11 @@ send_error_page(client_t *client)
             break;
     }
     client->keep_alive = 0;
+    client->header_done = 1;
+    client->response_closed = 1;
 }
 
 
-static inline void
-extent_sndbuf(client_t *client)
-{
-    int bufsize = 1024 * 1024 * 2, r;
-    r = setsockopt(client->fd, SOL_SOCKET, SO_SNDBUF, &bufsize, sizeof(bufsize));
-    assert(r == 0);
-}
-
-static inline void 
-enable_cork(client_t *client)
-{
-    int on = 1, r;
-    r = setsockopt(client->fd, IPPROTO_TCP, TCP_CORK, &on, sizeof(on));
-    assert(r == 0);
-}
-
 
 static inline write_bucket *
 new_write_bucket(int fd, int cnt){
@@ -350,12 +338,13 @@ write_sendfile(int out_fd, int in_fd, size_t count)
     return sendfile(out_fd, in_fd, NULL, count);
 }
 
-static inline void 
+inline void 
 close_response(client_t *client)
 {
+    
     //send all response
     //closing reponse object
-    if (PyObject_HasAttrString(client->response, "close")) {
+    if (client->response && PyObject_HasAttrString(client->response, "close")) {
         PyObject *close = NULL;
         PyObject *args = NULL;
         PyObject *data = NULL;
@@ -371,8 +360,8 @@ close_response(client_t *client)
         if (PyErr_Occurred()){
             PyErr_Clear();
         }
-        client->response_closed = 1; 
     }
+    client->response_closed = 1; 
 
 }
 
@@ -458,6 +447,13 @@ processs_write(client_t *client)
                         break;
                     }
                 }
+            }else{ 
+                PyErr_SetString(PyExc_TypeError, "response item must be a string");
+                Py_DECREF(item);
+                if (PyErr_Occurred()){ 
+                    write_error_log(__FILE__, __LINE__);
+                    return -1;
+                }
             }
             Py_DECREF(item);
         }
@@ -544,9 +540,6 @@ start_response_write(client_t *client)
     }
     client->response_iter = iterator;
 
-#ifdef DEBUG
-    printf("response_iter ob_refcnt %d \n", client->response_iter->ob_refcnt);
-#endif
     item =  PyIter_Next(iterator);
     if(item != NULL && PyString_Check(item)){
 
@@ -556,10 +549,17 @@ start_response_write(client_t *client)
 
 #ifdef DEBUG
         printf("start_response_write buflen %d \n", buflen);
-        //printf("item refcnt %d \n", item->ob_refcnt);
 #endif
         Py_DECREF(item);
         return write_headers(client, buf, buflen);
+    }else{
+        PyErr_SetString(PyExc_TypeError, "response item must be a string");
+        Py_XDECREF(item);
+        if (PyErr_Occurred()){ 
+            write_error_log(__FILE__, __LINE__);
+            return -1;
+        }
+        
     }
     return -1;
 }
@@ -698,6 +698,7 @@ ResponseObject_call(PyObject *obj, PyObject *args, PyObject *kw)
     Py_INCREF(self->cli->headers);
     
     Py_XDECREF(self->cli->http_status);
+    //self->cli->http_status =  PyString_FromFormat("HTTP/1.1 %s\r\n", PyString_AS_STRING(status));
     self->cli->http_status =  PyString_FromFormat("HTTP/1.0 %s\r\n", PyString_AS_STRING(status));
 
     Py_RETURN_NONE;
