@@ -12,6 +12,7 @@
 #include "response.h"
 #include "log.h"
 #include "client.h"
+#include "socket.h"
 
 #define MAX_FDS 1024 * 8
 #define ACCEPT_TIMEOUT_SECS 1
@@ -28,7 +29,9 @@ static short server_port = 8000;
 static int listen_sock;  // listen socket
 
 static int loop_done; // main loop flag
-static picoev_loop* main_loop; //main loop
+
+picoev_loop* main_loop; //main loop
+
 static PyObject *wsgi_app = NULL; //wsgi app
 
 static PyObject *watchdog = NULL; //watchdog
@@ -43,7 +46,7 @@ int max_content_length = 1024 * 1024 * 16; //max_content_length
 
 static char *unix_sock_name = NULL;
 
-static PyObject *switch_value;
+PyObject *hub_switch_value;
 
 
 static void
@@ -67,31 +70,8 @@ setsig(int sig, void* handler)
 }
 
 
-static inline void 
-setup_sock(int fd)
-{
-    int on = 1, r;
-    r = setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on));
-    assert(r == 0);
-    r = fcntl(fd, F_SETFL, O_NONBLOCK);
-    assert(r == 0);
-}
-
-
-static inline void 
-disable_cork(client_t *client)
-{
-    int off = 0;
-    int on = 1, r;
-    r = setsockopt(client->fd, IPPROTO_TCP, TCP_CORK, &off, sizeof(off));
-    assert(r == 0);
-
-    r = setsockopt(client->fd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on));
-    assert(r == 0);
-}
-
 static inline client_t *
-new_client_t(int client_fd, struct sockaddr_in client_addr){
+new_client_t(int client_fd, char *remote_addr, uint32_t remote_port){
     client_t *client;
     
     client = PyMem_Malloc(sizeof(client_t)); 
@@ -101,8 +81,8 @@ new_client_t(int client_fd, struct sockaddr_in client_addr){
 
     client->fd = client_fd;
     
-    client->remote_addr = inet_ntoa (client_addr.sin_addr);
-    client->remote_port = ntohs(client_addr.sin_port);
+    client->remote_addr = remote_addr;
+    client->remote_port = remote_port;
     client->req = new_request();    
     client->body_type = BODY_TYPE_NONE;
     //printf("input_buf_size %d\n", client->input_buf_size);
@@ -117,17 +97,17 @@ clean_cli(client_t *client)
         free_request(client->req);
         client->req = NULL;
     }
-    Py_XDECREF(client->http_status);
-    Py_XDECREF(client->headers);
-    Py_XDECREF(client->response_iter);
-    Py_XDECREF(client->response);
-    Py_XDECREF(client->environ);
+    Py_CLEAR(client->http_status);
+    Py_CLEAR(client->headers);
+    Py_CLEAR(client->response_iter);
+    Py_CLEAR(client->response);
+    Py_CLEAR(client->environ);
     if(client->body_type == BODY_TYPE_TMPFILE){
         if(client->body){
             fclose(client->body);
         }
     }else{
-        Py_XDECREF(client->body);
+        Py_CLEAR(client->body);
     }
     if(client->http != NULL){
         PyMem_Free(client->http);
@@ -139,33 +119,25 @@ clean_cli(client_t *client)
 static inline void 
 close_conn(client_t *cli, picoev_loop* loop)
 {
-    
+    client_t *new_client;
+    if(!cli->response_closed){
+        close_response(cli);
+    }
+
+    picoev_del(loop, cli->fd);
+    clean_cli(cli);
     if(!cli->keep_alive){
-        picoev_del(loop, cli->fd);
-        clean_cli(cli);
         close(cli->fd);
 #ifdef DEBUG
         printf("close fd %d \n", cli->fd);
 #endif
-        PyMem_Free(cli);
     }else{
-        clean_cli(cli);
         disable_cork(cli);
-        cli->keep_alive = 1;
-        cli->environ = NULL;
-        cli->http_status = NULL;
-        cli->headers = NULL;
-        cli->header_done = 0;
-        cli->body_type = BODY_TYPE_NONE;
-        cli->status_code = 0;
-        cli->response = NULL;
-        cli->content_length_set = 0;
-        cli->content_length = 0;
-        cli->write_bytes = 0;
-        cli->response_closed = 0;
-        cli->bad_request_code = 0;
-        init_parser(cli, server_name, server_port);
+        new_client = new_client_t(cli->fd, cli->remote_addr, cli->remote_port);
+        init_parser(new_client, server_name, server_port);
+        picoev_add(main_loop, new_client->fd, PICOEV_READ, READ_LONG_TIMEOUT_SECS, r_callback, (void *)new_client);
     }
+    PyMem_Free(cli);
 
 }
 
@@ -173,7 +145,7 @@ static inline int
 process_resume_wsgi_app(ClientObject *pyclient)
 {
     PyObject *res = NULL;
-
+    PyObject *err_type, *err_val, *err_tb;
     client_t *old_client;
     client_t *client = pyclient->client;
     
@@ -182,13 +154,21 @@ process_resume_wsgi_app(ClientObject *pyclient)
     old_client = start_response->cli;
     start_response->cli = client;
     
-    res = PyGreenlet_Switch(pyclient->greenlet, pyclient->args, pyclient->kwargs);
+    if(PyErr_Occurred()){
+        PyErr_Fetch(&err_type, &err_val, &err_tb);
+        PyErr_Clear();
+        //set error
+        res = PyGreenlet_Throw(pyclient->greenlet, err_type, err_val, err_tb);
+    }else{
+        res = PyGreenlet_Switch(pyclient->greenlet, pyclient->args, pyclient->kwargs);
+    }
     start_response->cli = old_client;
     
-    Py_XDECREF(pyclient->args);
-    Py_XDECREF(pyclient->kwargs);
+    Py_CLEAR(pyclient->args);
+    Py_CLEAR(pyclient->kwargs);
+    
 
-    //check response & Py_ErrorOccued
+    //check response & PyErr_Occurred
     if(res && res == Py_None){
         PyErr_SetString(PyExc_Exception, "response must be a iter or sequence object");
     }
@@ -199,8 +179,10 @@ process_resume_wsgi_app(ClientObject *pyclient)
     }
     
     if(PyInt_Check(res)){
-        // suspend process
-        return 0;
+        if(PyInt_AS_LONG(res) == -1){
+            // suspend process
+            return 0;
+        }
     }
 
     client->response = res;
@@ -225,7 +207,7 @@ process_wsgi_app(client_t *cli)
     Py_DECREF(args);
     
 
-    //check response & Py_ErrorOccued
+    //check response & PyErr_Occurred
     if(res && res == Py_None){
         PyErr_SetString(PyExc_Exception, "response must be a iter or sequence object");
     }
@@ -236,8 +218,10 @@ process_wsgi_app(client_t *cli)
     }
     
     if(PyInt_Check(res)){
-        // suspend process
-        return 0;
+        if(PyInt_AS_LONG(res) == -1){
+            // suspend process
+            return 0;
+        }
     }
 
     //next send response 
@@ -247,17 +231,25 @@ process_wsgi_app(client_t *cli)
     
 }
 
+inline void
+switch_wsgi_app(picoev_loop* loop, int fd, PyObject *obj)
+{
+    ClientObject *pyclient = (ClientObject *)obj;
+    picoev_del(loop, fd);
+    // resume
+    resume_wsgi_app(pyclient, loop);
+    pyclient->resumed = 0;
+}
+
 static void
 resume_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
     ClientObject *pyclient = (ClientObject *)(cb_arg);
     client_t *client = pyclient->client;
-    picoev_del(loop, client->fd);
-    // resume
-    resume_wsgi_app(pyclient, loop);
-    pyclient->resumed = 0;
+    switch_wsgi_app(loop, client->fd, (PyObject *)pyclient); 
 }
 
+
 static void
 w_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 {
@@ -309,6 +301,13 @@ resume_wsgi_app(ClientObject *pyclient, picoev_loop* loop)
             pyclient->client = NULL;
             break;
     }
+
+
+    if(client->response_closed){
+        //closed
+        close_conn(client, loop);
+        return;
+    }
     
     ret = response_start(client);
     switch(ret){
@@ -351,6 +350,12 @@ call_wsgi_app(client_t *client, picoev_loop* loop)
             break;
     }
     
+    
+    if(client->response_closed){
+        //closed
+        close_conn(client, loop);
+        return;
+    }
     ret = response_start(client);
     switch(ret){
         case -1:
@@ -396,7 +401,11 @@ prepare_call_wsgi(client_t *client)
             input = PycStringIO->NewInput(object);
             PyDict_SetItemString((PyObject *)client->environ, "wsgi.input", input);
         }else{
-            object = PyString_FromString("");
+            if(client->body){
+                object = client->body;
+            }else{
+                object = PyString_FromString("");
+            }
             input = PycStringIO->NewInput(object);
             PyDict_SetItemString((PyObject *)client->environ, "wsgi.input", input);
         }
@@ -427,7 +436,7 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
     if ((events & PICOEV_TIMEOUT) != 0) {
 
 #ifdef DEBUG
-        printf("** r_callback timeout ** \n");
+        printf("** r_callback timeout %d ** \n", fd);
 #endif
         //timeout
         cli->keep_alive = 0;
@@ -464,17 +473,24 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                     write_error_log(__FILE__, __LINE__); 
                     cli->keep_alive = 0;
                     cli->status_code = 500;
-                    send_error_page(cli);
+                    if(errno != ECONNRESET){
+                        send_error_page(cli);
+                    }else{
+
+                        cli->header_done = 1;
+                        cli->response_closed = 1;
+                    }
                     close_conn(cli, loop);
                     return;
                 }
                 break;
             default:
 
-#ifdef DEBUG
-                printf("read request fd %d bufsize %d \n", cli->fd, r);
-#endif
                 nread = execute_parse(cli, buf, r);
+#ifdef DEBUG
+                printf("read request fd %d readed %d nread %d \n", cli->fd, r, nread);
+                printf("%s\n", buf);
+#endif
                 
                 if(cli->bad_request_code > 0){
 #ifdef DEBUG
@@ -484,7 +500,14 @@ r_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
                     close_conn(cli, loop);
                     return;
                 }
-                if( nread != r ){
+
+                if(cli->upgrade){
+                    //WebSocket Key
+                    char *key = buf + nread + 1;
+                    PyObject *body = PyString_FromStringAndSize(key, r - nread);
+                    cli->body = body;
+
+                }else if(!cli->upgrade && nread != r){
                     // parse error
 #ifdef DEBUG
                     printf("fd %d parse error %d \n", cli->fd, cli->bad_request_code);
@@ -521,6 +544,8 @@ accept_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
     int client_fd;
     client_t *client;
     struct sockaddr_in client_addr;
+    char *remote_addr;
+    uint32_t remote_port;
     if ((events & PICOEV_TIMEOUT) != 0) {
         // time out
         // next turn or other process
@@ -536,7 +561,9 @@ accept_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 #endif
             //printf("connected: %d\n", client_fd);
             setup_sock(client_fd);
-            client = new_client_t(client_fd, client_addr);
+            remote_addr = inet_ntoa (client_addr.sin_addr);
+            remote_port = ntohs(client_addr.sin_port);
+            client = new_client_t(client_fd, remote_addr, remote_port);
             init_parser(client, server_name, server_port);
             picoev_add(loop, client_fd, PICOEV_READ, READ_LONG_TIMEOUT_SECS, r_callback, (void *)client);
         }else{
@@ -554,14 +581,15 @@ accept_callback(picoev_loop* loop, int fd, int events, void* cb_arg)
 static inline void
 setup_server_env(void)
 {
-    setup_sock(listen_sock);
+    setup_listen_sock(listen_sock);
     cache_time_init();
     setup_static_env(server_name, server_port);
     setup_start_response();
     setup_client();
+    setup_nsocket();
     PycString_IMPORT;
     PyGreenlet_Import();
-    switch_value = Py_BuildValue("(i)", -1);
+    hub_switch_value = Py_BuildValue("(i)", -1);
 }
 
 static inline int 
@@ -828,8 +856,12 @@ meinheld_run_loop(PyObject *self, PyObject *args)
     picoev_destroy_loop(main_loop);
     picoev_deinit();
     
+    //clean
     clear_start_response();
     clear_static_env();
+
+    Py_DECREF(hub_switch_value);
+
     if(unix_sock_name){
         unlink(unix_sock_name);
     }
@@ -953,7 +985,7 @@ meinheld_suspend_client(PyObject *self, PyObject *args)
     if(pyclient->client && !(pyclient->suspended)){
         pyclient->suspended = 1;
         parent = PyGreenlet_GET_PARENT(pyclient->greenlet);
-        return PyGreenlet_Switch(parent, switch_value, NULL);
+        return PyGreenlet_Switch(parent, hub_switch_value, NULL);
     }else{
         PyErr_SetString(PyExc_Exception, "already suspended");
         return NULL;
@@ -992,14 +1024,11 @@ meinheld_resume_client(PyObject *self, PyObject *args)
     }
 
     if(pyclient->client && !pyclient->resumed){
-        if(switch_args){
-            pyclient->args = switch_args;
-            Py_INCREF(pyclient->args);
-        }
-        if(switch_kwargs){
-            pyclient->kwargs = switch_kwargs;
-            Py_INCREF(pyclient->kwargs);
-        }
+        pyclient->args = switch_args;
+        Py_XINCREF(pyclient->args);
+    
+        pyclient->kwargs = switch_kwargs;
+        Py_XINCREF(pyclient->kwargs);
         client = pyclient->client;
 
         pyclient->suspended = 0;
@@ -1012,6 +1041,23 @@ meinheld_resume_client(PyObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
+PyObject *
+meinheld_get_socket_fromfd(PyObject *self, PyObject *args)
+{
+    int fd;
+    PyObject *temp;
+    if (!PyArg_ParseTuple(args, "iO:_get_socket_fromfd", &fd, &temp)){
+        return NULL;
+    }
+    if(!CheckClientObject(temp)){
+        PyErr_SetString(PyExc_TypeError, "must be a client object");
+        return NULL;
+    }
+    return NSocketObject_New(fd, (ClientObject *)temp);
+
+}
+
+
 static PyMethodDef WsMethods[] = {
     {"listen", meinheld_listen, METH_VARARGS, "set host and port num"},
     {"access_log", meinheld_access_log, METH_VARARGS, "set access log file path."},
@@ -1026,9 +1072,11 @@ static PyMethodDef WsMethods[] = {
     {"set_listen_socket", meinheld_set_listen_socket, METH_VARARGS, "set listen_sock"},
     {"set_watchdog", meinheld_set_watchdog, METH_VARARGS, "set watchdog"},
     {"run", meinheld_run_loop, METH_VARARGS, "set wsgi app, run the main loop"},
-    // green
+    // greenlet and continuation
     {"_suspend_client", meinheld_suspend_client, METH_VARARGS, "resume client"},
     {"_resume_client", meinheld_resume_client, METH_VARARGS, "resume client"},
+    // io
+    {"_get_socket_fromfd", meinheld_get_socket_fromfd, METH_VARARGS, "get socket fromfd"},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
@@ -1038,6 +1086,7 @@ initserver(void)
     (void)Py_InitModule("meinheld.server", WsMethods);
     PyType_Ready(&FileWrapperType);
     PyType_Ready(&ClientObjectType);
+    PyType_Ready(&NSocketObjectType);
 
 }
 
