@@ -19,13 +19,13 @@
 #include <netinet/tcp.h>
 #include <netdb.h>
 
+#include "greenlet.h"
 #include "picoev.h"
 #include "request.h"
 #include "http_parser.h"
 
-#include "greenlet.h"
 
-#define SERVER "meinheld/0.2"
+#define SERVER "meinheld/0.2.1"
 
 typedef enum {
     BODY_TYPE_NONE,
@@ -45,6 +45,7 @@ typedef struct _client {
     void *body;
     int bad_request_code;
     request_body_type body_type;    
+    char upgrade;               // new protocol
     uint8_t complete;
 
     http_parser *http;          // http req parser
@@ -65,5 +66,11 @@ typedef struct _client {
 
 extern int max_content_length; //max_content_length
 
+extern picoev_loop* main_loop; //main loop
+
+extern PyObject* hub_switch_value;
+
+inline void
+switch_wsgi_app(picoev_loop* loop, int fd, PyObject *obj);
 
 #endif
